@@ -17,10 +17,10 @@ typedef struct config_t
 	bool transDel, fullScreenStretch, vsyncOff, modDot, blankZeroFlag, realVuMeters, rememberPlayMode;
 	bool startInFullscreen, integerScaling, disableE8xEffect, noDownsampleOnSmpLoad, keepEditModeAfterStepPlay;
 	int8_t stereoSeparation, videoScaleFactor, accidental;
-	uint8_t pixelFilter, filterModel;
+	uint8_t pixelFilter, amigaModel;
 	uint16_t quantizeValue;
 	int32_t maxSampleLength;
-	uint32_t soundFrequency, soundBufferSize, audioInputFrequency, reservedSampleOffset;
+	uint32_t soundFrequency, soundBufferSize, audioInputFrequency, mod2WavOutputFreq;
 } config_t;
 
 extern config_t config; // pt2_config.c
