@@ -23,19 +23,16 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <limits.h>
-#include "pt2_header.h"
 #include "pt2_textout.h"
 #include "pt2_diskop.h"
 #include "pt2_tables.h"
 #include "pt2_module_loader.h"
-#include "pt2_audio.h"
-#include "pt2_sampler.h"
 #include "pt2_config.h"
-#include "pt2_helpers.h"
-#include "pt2_keyboard.h"
 #include "pt2_visuals.h"
 #include "pt2_sample_loader.h"
 #include "pt2_bmp.h"
+#include "pt2_askbox.h"
+#include "pt2_replayer.h"
 
 typedef struct fileEntry_t
 {
@@ -63,7 +60,6 @@ static DIR *hFind;
 #endif
 
 static char fileNameBuffer[PATH_MAX + 1];
-static uint32_t oldFileEntryRow;
 static UNICHAR pathTmp[PATH_MAX + 2];
 static fileEntry_t *diskOpEntry;
 
@@ -79,9 +75,7 @@ void addSampleFileExt(char *fileName)
 
 static fileEntry_t *bufferCreateEmptyDir(void) // special case: creates a dir entry with a ".." directory
 {
-	fileEntry_t *dirEntry;
-
-	dirEntry = (fileEntry_t *)malloc(sizeof (fileEntry_t));
+	fileEntry_t *dirEntry = (fileEntry_t *)malloc(sizeof (fileEntry_t));
 	if (dirEntry == NULL)
 		return NULL;
 
@@ -283,7 +277,7 @@ static void findClose(void)
 
 void diskOpShowSelectText(void)
 {
-	if (!ui.diskOpScreenShown || ui.pointerMode == POINTER_MODE_MSG1 || editor.errorMsgActive)
+	if (ui.pointerMode == POINTER_MODE_MSG1 || editor.errorMsgActive)
 		return;
 
 	if (diskop.mode == DISKOP_MODE_MOD)
@@ -340,11 +334,9 @@ bool diskOpEntryIsDir(int32_t fileIndex)
 
 char *diskOpGetAnsiEntry(int32_t fileIndex)
 {
-	UNICHAR *filenameU;
-
 	if (diskOpEntry != NULL && !diskOpEntryIsEmpty(fileIndex))
 	{
-		filenameU = diskOpEntry[diskop.scrollOffset+fileIndex].nameU;
+		UNICHAR *filenameU = diskOpEntry[diskop.scrollOffset+fileIndex].nameU;
 		if (filenameU != NULL)
 		{
 			unicharToAnsi(fileNameBuffer, filenameU, PATH_MAX);
@@ -378,12 +370,11 @@ static void setVisualPathToCwd(void)
 	ui.updateDiskOpPathText = true;
 }
 
-bool changePathToHome(void)
+bool changePathToDesktop(void)
 {
 #ifdef _WIN32
 	UNICHAR pathU[PATH_MAX + 2];
-
-	if (SHGetFolderPathW(NULL, CSIDL_PROFILE, NULL, 0, pathU) >= 0)
+	if (SHGetFolderPathW(NULL, CSIDL_DESKTOPDIRECTORY, NULL, 0, pathU) >= 0)
 	{
 		if (UNICHAR_CHDIR(pathU) == 0)
 			return true;
@@ -391,16 +382,28 @@ bool changePathToHome(void)
 
 	return false;
 #else
-	char *homePath;
-
-	homePath = getenv("HOME");
+	char *homePath = getenv("HOME");
 	if (homePath != NULL && chdir(homePath) == 0)
+	{
+		chdir("Desktop"); // keep home dir in case we couldn't change the dir to desktop
 		return true;
+	}
 
 	return false;
 #endif
 }
 
+#ifndef _WIN32
+bool changePathToHome(void)
+{
+	char *homePath = getenv("HOME");
+	if (homePath != NULL && chdir(homePath) == 0)
+		return true;
+
+	return false;
+}
+#endif
+
 void setPathFromDiskOpMode(void)
 {
 	UNICHAR_CHDIR((diskop.mode == DISKOP_MODE_MOD) ? editor.modulesPathU : editor.samplesPathU);
@@ -556,8 +559,7 @@ static void sortEntries(void)
 
 static bool diskOpFillBuffer(void)
 {
-	uint8_t lastFindFileFlag;
-	fileEntry_t tmpBuffer, *newPtr;
+	fileEntry_t tmpBuffer;
 
 	diskop.scrollOffset = 0;
 
@@ -570,7 +572,7 @@ static bool diskOpFillBuffer(void)
 	// fill disk op. buffer (type, size, path, file name, date changed)
 
 	// read first file
-	lastFindFileFlag = findFirst(&tmpBuffer);
+	uint8_t lastFindFileFlag = findFirst(&tmpBuffer);
 	if (lastFindFileFlag != LFF_DONE && lastFindFileFlag != LFF_SKIP)
 	{
 		diskOpEntry = (fileEntry_t *)malloc(sizeof (fileEntry_t) * (diskop.numEntries + 1));
@@ -592,7 +594,7 @@ static bool diskOpFillBuffer(void)
 		lastFindFileFlag = findNext(&tmpBuffer);
 		if (lastFindFileFlag != LFF_DONE && lastFindFileFlag != LFF_SKIP)
 		{
-			newPtr = (fileEntry_t *)realloc(diskOpEntry, sizeof (fileEntry_t) * (diskop.numEntries + 1));
+			fileEntry_t *newPtr = (fileEntry_t *)realloc(diskOpEntry, sizeof (fileEntry_t) * (diskop.numEntries + 1));
 			if (newPtr == NULL)
 			{
 				findClose();
@@ -641,7 +643,6 @@ static int32_t SDLCALL diskOpFillThreadFunc(void *ptr)
 static void printFileSize(fileEntry_t *entry, uint16_t x, uint16_t y)
 {
 	char tmpStr[7];
-	uint32_t fileSize, j;
 
 	if (entry->filesize == -1) // -1 means that the original filesize is above 2GB in our directory reader
 	{
@@ -649,7 +650,7 @@ static void printFileSize(fileEntry_t *entry, uint16_t x, uint16_t y)
 		return;
 	}
 
-	fileSize = (uint32_t)entry->filesize;
+	uint32_t fileSize = (uint32_t)entry->filesize;
 	if (fileSize <= 999999)
 	{
 		// bytes
@@ -669,12 +670,12 @@ static void printFileSize(fileEntry_t *entry, uint16_t x, uint16_t y)
 	}
 
 	// turn zeroes on the left side into spaces
-	for (j = 0; j < 7; j++)
+	for (int32_t i = 0; i < 7; i++)
 	{
-		if (tmpStr[j] != '0')
+		if (tmpStr[i] != '0')
 			break;
 
-		tmpStr[j] = ' ';
+		tmpStr[i] = ' ';
 	}
 
 	textOut(x, y, tmpStr, video.palette[PAL_QADSCP]);
@@ -685,8 +686,8 @@ static void printEntryName(char *entryName, int32_t entryLength, int32_t maxLeng
 	if (entryLength > maxLength)
 	{
 		// shorten name and add ".." to end
-		for (int32_t j = 0; j < maxLength-2; j++)
-			charOut(x + (j * FONT_CHAR_W), y, entryName[j], video.palette[PAL_QADSCP]);
+		for (int32_t i = 0; i < maxLength-2; i++)
+			charOut(x + (i * FONT_CHAR_W), y, entryName[i], video.palette[PAL_QADSCP]);
 
 		textOut(x + ((maxLength - 2) * FONT_CHAR_W), y, "..", video.palette[PAL_QADSCP]);
 	}
@@ -699,11 +700,8 @@ static void printEntryName(char *entryName, int32_t entryLength, int32_t maxLeng
 
 void diskOpRenderFileList(void)
 {
-	char *entryName;
 	uint8_t maxFilenameChars, maxDirNameChars;
-	uint16_t x, y, textXStart;
-	int32_t entryLength;
-	fileEntry_t *entry;
+	uint16_t textXStart;
 
 	if (config.hideDiskOpDates)
 	{
@@ -727,8 +725,8 @@ void diskOpRenderFileList(void)
 	if (!diskop.cached)
 	{
 		diskop.fillThread = SDL_CreateThread(diskOpFillThreadFunc, NULL, NULL);
-		if (diskop.fillThread != NULL)
-			SDL_DetachThread(diskop.fillThread);
+		if (diskop.fillThread == NULL)
+			return;
 
 		diskop.cached = true;
 		return;
@@ -746,12 +744,12 @@ void diskOpRenderFileList(void)
 		if (diskop.scrollOffset+i >= diskop.numEntries)
 			break;
 
-		entry = &diskOpEntry[diskop.scrollOffset+i];
-		entryName = diskOpGetAnsiEntry(i);
-		entryLength = (int32_t)strlen(entryName);
+		fileEntry_t *entry = &diskOpEntry[diskop.scrollOffset+i];
+		char *entryName = diskOpGetAnsiEntry(i);
+		int32_t entryLength = (int32_t)strlen(entryName);
 
-		x = textXStart;
-		y = (uint8_t)(35 + (i * (FONT_CHAR_H + 1)));
+		uint16_t x = textXStart;
+		uint16_t y = (uint8_t)(35 + (i * (FONT_CHAR_H + 1)));
 
 		if (!entry->isDir)
 		{
@@ -774,9 +772,6 @@ void diskOpRenderFileList(void)
 
 void diskOpLoadFile(uint32_t fileEntryRow, bool songModifiedCheck)
 {
-	uint8_t oldMode, oldPlayMode;
-	UNICHAR *filePath;
-
 	// if we clicked on an empty space, return...
 	if (diskOpEntryIsEmpty(fileEntryRow))
 		return;
@@ -787,23 +782,22 @@ void diskOpLoadFile(uint32_t fileEntryRow, bool songModifiedCheck)
 	}
 	else
 	{
-		filePath = diskOpGetUnicodeEntry(fileEntryRow);
+		UNICHAR *filePath = diskOpGetUnicodeEntry(fileEntryRow);
 		if (filePath != NULL)
 		{
 			if (diskop.mode == DISKOP_MODE_MOD)
 			{
 				if (songModifiedCheck && song->modified)
 				{
-					oldFileEntryRow = fileEntryRow;
-					showSongUnsavedAskBox(ASK_DISCARD_SONG);
-					return;
+					if (!askBox(ASKBOX_YES_NO, "SONG IS UNSAVED !"))
+						return;
 				}
 
 				module_t *newSong = modLoad(filePath);
 				if (newSong != NULL)
 				{
-					oldMode = editor.currMode;
-					oldPlayMode = editor.playMode;
+					uint8_t oldMode = editor.currMode;
+					uint8_t oldPlayMode = editor.playMode;
 
 					modStop();
 					modFree();
@@ -865,11 +859,6 @@ void diskOpLoadFile(uint32_t fileEntryRow, bool songModifiedCheck)
 	}
 }
 
-void diskOpLoadFile2(void)
-{
-	diskOpLoadFile(oldFileEntryRow, false);
-}
-
 void renderDiskOpScreen(void)
 {
 	blit32(0, 0, 320, 99, diskOpScreenBMP);
@@ -883,9 +872,7 @@ void renderDiskOpScreen(void)
 
 void updateDiskOp(void)
 {
-	char tmpChar;
-
-	if (!ui.diskOpScreenShown || ui.posEdScreenShown)
+	if (!ui.diskOpScreenShown || ui.posEdScreenShown || ui.askBoxShown)
 		return;
 
 	if (ui.updateDiskOpFileList)
@@ -930,7 +917,7 @@ void updateDiskOp(void)
 		// print disk op. path
 		for (int32_t i = 0; i < 26; i++)
 		{
-			tmpChar = editor.currPath[ui.diskOpPathTextOffset+i];
+			char tmpChar = editor.currPath[ui.diskOpPathTextOffset+i];
 			if (tmpChar == '\0')
 				tmpChar = '_';
 
