@@ -17,7 +17,6 @@
 #include <unistd.h> // chdir()
 #endif
 #include <sys/stat.h>
-#include "pt2_header.h"
 #include "pt2_helpers.h"
 #include "pt2_keyboard.h"
 #include "pt2_textout.h"
@@ -29,14 +28,13 @@
 #include "pt2_edit.h"
 #include "pt2_module_loader.h"
 #include "pt2_module_saver.h"
-#include "pt2_sample_loader.h"
-#include "pt2_unicode.h"
 #include "pt2_scopes.h"
 #include "pt2_audio.h"
 #include "pt2_bmp.h"
-#include "pt2_sync.h"
+#include "pt2_visuals_sync.h"
 #include "pt2_sampling.h"
-#include "pt2_hpc.h"
+#include "pt2_askbox.h"
+#include "pt2_replayer.h"
 
 #define CRASH_TEXT "Oh no!\nThe ProTracker 2 clone has crashed...\n\nA backup .mod was hopefully " \
                    "saved to the current module directory.\n\nPlease report this bug if you can.\n" \
@@ -118,10 +116,16 @@ int main(int argc, char *argv[])
 	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
 #endif
 
-#if SDL_PATCHLEVEL < 5
-	#pragma message("WARNING: The SDL2 dev lib is older than ver 2.0.5. You'll get fullscreen mode issues.")
+#if SDL_MAJOR_VERSION == 2 && SDL_MINOR_VERSION == 0 && SDL_PATCHLEVEL < 5
+#pragma message("WARNING: The SDL2 dev lib is older than ver 2.0.5. You'll get fullscreen mode issues and no audio input sampling.")
+#pragma message("At least version 2.0.7 is recommended.")
 #endif
 
+	SDL_SetThreadPriority(SDL_THREAD_PRIORITY_HIGH);
+	SDL_EnableScreenSaver(); // allow screensaver to activate
+
+	clearStructs();
+
 	// set up crash handler
 #ifndef _DEBUG
 #ifdef _WIN32
@@ -139,8 +143,6 @@ int main(int argc, char *argv[])
 #endif
 #endif
 
-	clearStructs();
-
 	// on Windows and macOS, test what version SDL2.DLL is (against library version used in compilation)
 #if defined _WIN32 || defined __APPLE__
 	SDL_GetVersion(&sdlVer);
@@ -165,13 +167,18 @@ int main(int argc, char *argv[])
 
 #ifdef _WIN32
 
+	// ALT+F4 is used in ProTracker, but is "close program" in Windows...
+#if SDL_MINOR_VERSION >= 24 || (SDL_MINOR_VERSION == 0 && SDL_PATCHLEVEL >= 4)
+	SDL_SetHint(SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4, "1");
+#endif
+
 #ifndef _MSC_VER
 	SetProcessDPIAware();
 #endif
 
 	if (!SDL_HasSSE())
 	{
-		showErrorMsgBox("Your computer's processor doesn't have the SSE+SSE2 instruction set\n" \
+		showErrorMsgBox("Your computer's processor doesn't have the SSE instruction set\n" \
 		                "which is needed for this program to run. Sorry!");
 		return 0;
 	}
@@ -192,7 +199,7 @@ int main(int argc, char *argv[])
 	** reinitialized in Windows and what not.
 	** Ref.: https://bugzilla.libsdl.org/show_bug.cgi?id=4391
 	*/
-#if defined _WIN32 && SDL_PATCHLEVEL == 9
+#if defined _WIN32 && SDL_MAJOR_VERSION == 2 && SDL_MINOR_VERSION == 0 && SDL_PATCHLEVEL == 9
 	if (SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) != 0)
 #else
 	if (SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO) != 0)
@@ -202,9 +209,6 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
-	SDL_SetThreadPriority(SDL_THREAD_PRIORITY_HIGH);
-	SDL_EnableScreenSaver(); // allow screensaver to activate
-
 	/* Text input is started by default in SDL2, turn it off to remove ~2ms spikes per key press.
 	** We manuallay start it again when someone clicks on a text edit box, and stop it when done.
 	** Ref.: https://bugzilla.libsdl.org/show_bug.cgi?id=4166
@@ -326,14 +330,18 @@ int main(int argc, char *argv[])
 	if (config.startInFullscreen)
 		toggleFullscreen();
 
-	changePathToHome(); // set path to home/user-dir now
+	changePathToDesktop(); // change path to desktop now
 	diskOpSetInitPath(); // set path to custom path in config (if present)
 
 	SDL_EventState(SDL_DROPFILE, SDL_ENABLE);
 
-	hpc_ResetEndTime(&video.vblankHpc);
+	editor.mainLoopOngoing = true;
+	hpc_ResetCounters(&video.vblankHpc); // this must be the last thing we do before entering the main loop
+
+	// XXX: if you change anything in the main loop, make sure it goes in the askBox()(pt2_askbox.c) loop too, if needed
 	while (editor.programRunning)
 	{
+		handleThreadedAskBox();
 		sinkVisualizerBars();
 		updateChannelSyncBuffer();
 		readMouseXY();
@@ -342,11 +350,8 @@ int main(int argc, char *argv[])
 		updateMouseCounters();
 		handleKeyRepeat(keyb.lastRepKey);
 
-		if (!mouse.buttonWaiting && ui.sampleMarkingPos == -1 &&
-			!ui.forceSampleDrag && !ui.forceVolDrag && !ui.forceSampleEdit)
-		{
+		if (!mouse.buttonWaiting && ui.sampleMarkingPos == -1 && !ui.forceSampleDrag && !ui.forceVolDrag && !ui.forceSampleEdit)
 			handleGUIButtonRepeat();
-		}
 
 		renderFrame();
 		flipFrame();
@@ -360,9 +365,7 @@ int main(int argc, char *argv[])
 
 static void handleInput(void)
 {
-	char inputChar;
 	SDL_Event event;
-
 	while (SDL_PollEvent(&event))
 	{
 		if (event.type == SDL_WINDOWEVENT)
@@ -374,7 +377,7 @@ static void handleInput(void)
 
 			// reset vblank end time if we minimize window
 			if (event.window.event == SDL_WINDOWEVENT_MINIMIZED || event.window.event == SDL_WINDOWEVENT_FOCUS_LOST)
-				hpc_ResetEndTime(&video.vblankHpc);
+				hpc_ResetCounters(&video.vblankHpc);
 		}
 
 #ifdef _WIN32
@@ -384,7 +387,7 @@ static void handleInput(void)
 		{
 			// text input when editing texts/numbers
 
-			inputChar = event.text.text[0];
+			char inputChar = event.text.text[0];
 			if (inputChar == '\0')
 				continue;
 
@@ -398,12 +401,12 @@ static void handleInput(void)
 		}
 		else if (event.type == SDL_DROPFILE)
 		{
-			loadDroppedFile(event.drop.file, (uint32_t)strlen(event.drop.file), false, true);
-			SDL_free(event.drop.file);
-
 			// kludge: allow focus-clickthrough after drag-n-drop
 			SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "1");
 			didDropFile = true;
+
+			loadDroppedFile(event.drop.file, (uint32_t)strlen(event.drop.file), false, true);
+			SDL_free(event.drop.file);
 		}
 		if (event.type == SDL_QUIT)
 		{
@@ -411,7 +414,7 @@ static void handleInput(void)
 		}
 		else if (event.type == SDL_KEYUP)
 		{
-			keyUpHandler(event.key.keysym.scancode, event.key.keysym.sym);
+			keyUpHandler(event.key.keysym.scancode);
 		}
 		else if (event.type == SDL_KEYDOWN)
 		{
@@ -422,12 +425,12 @@ static void handleInput(void)
 		{
 			mouseButtonUpHandler(event.button.button);
 
-			if (!ui.askScreenShown && ui.introScreenShown)
+			if (ui.introTextShown)
 			{
-				if (!ui.clearScreenShown && !ui.diskOpScreenShown && !editor.errorMsgActive)
+				if (!ui.diskOpScreenShown && !editor.errorMsgActive && !ui.askBoxShown)
 					statusAllRight();
 
-				ui.introScreenShown = false;
+				ui.introTextShown = false;
 			}
 
 			// kludge: we drag-n-dropped a file before this mouse click release, restore focus-clickthrough mode
@@ -455,9 +458,10 @@ static void handleInput(void)
 				SDL_WaitThread(diskop.fillThread, NULL);
 			}
 
-			if (editor.isWAVRendering)
+			if (editor.mod2WavOngoing)
 			{
-				editor.isWAVRendering = false;
+				editor.mod2WavOngoing = false;
+
 				editor.abortMod2Wav = true;
 				SDL_WaitThread(editor.mod2WavThread, NULL);
 			}
@@ -530,7 +534,7 @@ static bool initializeVars(void)
 	editor.multiModeNext[1] = 3;
 	editor.multiModeNext[2] = 4;
 	editor.multiModeNext[3] = 1;
-	ui.introScreenShown = true;
+	ui.introTextShown = true;
 	editor.normalizeFiltersFlag = true;
 	editor.markStartOfs = -1;
 	ui.sampleMarkingPos = -1;
@@ -551,6 +555,8 @@ static bool initializeVars(void)
 	editor.metroChannelDisp = &editor.metroChannel;
 	editor.quantizeValueDisp = &config.quantizeValue;
 
+	editor.mod2WavFadeOutSeconds = 6;
+
 	editor.programRunning = true;
 	return true;
 
@@ -561,6 +567,15 @@ oom:
 
 static void handleSigTerm(void)
 {
+	if (editor.mod2WavOngoing)
+	{
+		editor.mod2WavOngoing = false;
+
+		editor.abortMod2Wav = true;
+		SDL_WaitThread(editor.mod2WavThread, NULL);
+		removeAskBox();
+	}
+
 	if (song->modified)
 	{
 		resetAllScreens();
@@ -572,12 +587,8 @@ static void handleSigTerm(void)
 			SDL_RaiseWindow(video.window);
 		}
 
-		ui.askScreenShown = true;
-		ui.askScreenType = ASK_QUIT;
-
-		pointerSetMode(POINTER_MODE_MSG1, NO_CARRY);
-		setStatusMessage("REALLY QUIT ?", NO_CARRY);
-		renderAskDialog();
+		if (askBox(ASKBOX_YES_NO, "REALLY QUIT ?"))
+			ui.throwExit = true;
 	}
 	else
 	{
@@ -589,21 +600,19 @@ static void handleSigTerm(void)
 #ifdef __APPLE__
 static void osxSetDirToProgramDirFromArgs(char **argv)
 {
-	char *tmpPath;
-	int32_t i, tmpPathLen;
-
 	/* OS X/macOS: hackish way of setting the current working directory to the place where we double clicked
-	** on the icon (for protracker.ini loading) */
+	** on the icon (for protracker.ini loading)
+	*/
 
 	// if we launched from the terminal, argv[0][0] would be '.'
 	if (argv[0] != NULL && argv[0][0] == DIR_DELIMITER) // don't do the hack if we launched from the terminal
 	{
-		tmpPath = strdup(argv[0]);
+		char *tmpPath = strdup(argv[0]);
 		if (tmpPath != NULL)
 		{
 			// cut off program filename
-			tmpPathLen = strlen(tmpPath);
-			for (i = tmpPathLen-1; i >= 0; i--)
+			int32_t tmpPathLen = strlen(tmpPath);
+			for (int32_t i = tmpPathLen-1; i >= 0; i--)
 			{
 				if (tmpPath[i] == DIR_DELIMITER)
 				{
@@ -684,20 +693,20 @@ static void disableWasapi(void)
 static void makeSureDirIsProgramDir(void)
 {
 #ifndef _DEBUG
-	UNICHAR *allocPtr, *path;
-	int32_t i, pathLen;
+	int32_t i;
 
 	// this can return two paths in Windows, but first one is .exe path
-	path = GetCommandLineW();
+	UNICHAR *path = GetCommandLineW();
 	if (path == NULL)
 		return;
 
-	allocPtr = UNICHAR_STRDUP(path);
+	UNICHAR *allocPtr = UNICHAR_STRDUP(path);
 	if (allocPtr == NULL)
 		return; // out of memory (but it doesn't matter)
 
 	path = allocPtr;
-	pathLen = (int32_t)UNICHAR_STRLEN(path);
+
+	int32_t pathLen = (int32_t)UNICHAR_STRLEN(path);
 
 	// remove first "
 	if (path[0] == L'\"')
