@@ -14,20 +14,15 @@ extern const char *noteNames2[2+36];
 extern const char *noteNames3[2+36];
 extern const char *noteNames4[2+36];
 extern const uint8_t vibratoTable[32];
-extern const uint8_t arpTickTable[32];
 extern const int16_t periodTable[(37*16)+15];
 extern int8_t pNoteTable[32];
+extern const uint64_t musicTimeTab64[(256-32)+1];
 
 // changable by config file
 extern uint16_t analyzerColors[36];
 extern uint16_t vuMeterColors[48];
 
-// button tables taken from the ptplay project + modified
-
 // MODIFY THESE EVERY TIME YOU REMOVE/ADD A BUTTON!
-#define ASK_BUTTONS 2
-#define PAT2SMP_ASK_BUTTONS 3
-#define CLEAR_BUTTONS 4
 #define TOPSCREEN_BUTTONS 47
 #define MIDSCREEN_BUTTONS 3
 #define BOTSCREEN_BUTTONS 4
@@ -40,9 +35,6 @@ extern uint16_t vuMeterColors[48];
 #define SAMPLER_BUTTONS 25
 // -----------------------------------------------
 
-extern const guiButton_t bAsk[];
-extern const guiButton_t bPat2SmpAsk[];
-extern const guiButton_t bClear[];
 extern const guiButton_t bTopScreen[];
 extern const guiButton_t bMidScreen[];
 extern const guiButton_t bBotScreen[];
