@@ -25,8 +25,10 @@ void hLine(int32_t x, int32_t y, int32_t w, const uint32_t pixelColor);
 void vLine(int32_t x, int32_t y, int32_t h, const uint32_t pixelColor);
 void drawFramework1(int32_t x, int32_t y, int32_t w, int32_t h);
 void drawFramework2(int32_t x, int32_t y, int32_t w, int32_t h);
+void drawFramework3(int32_t x, int32_t y, int32_t w, int32_t h);
 void fillRect(int32_t x, int32_t y, int32_t w, int32_t h, const uint32_t pixelColor);
-void drawButton(int32_t x, int32_t y, int32_t w, const char *text);
+void drawButton1(int32_t x, int32_t y, int32_t w, int32_t h, const char *text);
+void drawButton2(int32_t x, int32_t y, int32_t w, int32_t h, const char *text);
 void drawUpButton(int32_t x, int32_t y);
 void drawDownButton(int32_t x, int32_t y);
 
@@ -35,11 +37,10 @@ void statusOutOfMemory(void);
 void statusSampleIsEmpty(void);
 void statusNotSampleZero(void);
 void changeStatusText(const char *text);
-
 void resetAllScreens(void);
-void handleAskNo(void);
-void handleAskYes(void);
+
 bool setupVideo(void);
+void renderFrame2(void);
 void renderFrame(void);
 void flipFrame(void);
 void updateSpectrumAnalyzer(int8_t vol, int16_t period);
@@ -49,28 +50,20 @@ void updateVisualizer(void);
 void updateEditOp(void);
 void toggleFullscreen(void);
 void videoClose(void);
-
 void displayMainScreen(void);
-void renderAskDialog(void);
-void renderBigAskDialog(void);
-void showDownsampleAskDialog(void);
 void renderPosEdScreen(void);
 void renderMuteButtons(void);
-void renderClearScreen(void);
 void renderAboutScreen(void);
 void renderQuadrascopeBg(void);
 void renderSpectrumAnalyzerBg(void);
-void renderMOD2WAVDialog(void);
 void renderEditOpMode(void);
 void renderTextEditMarker(void);
 void renderEditOpScreen(void);
 void renderSamplerVolBox(void);
 void renderSamplerFiltersBox(void);
 void removeTextEditMarker(void);
-void removeClearScreen(void);
 void removeSamplerVolBox(void);
 void removeSamplerFiltersBox(void);
-void removeAskDialog(void);
 void fillToVuMetersBgBuffer(void);
 void showVolFromSlider(void);
 void showVolToSlider(void);
